@@ -1,4 +1,4 @@
-/* $OpenBSD: ssl_lib.c,v 1.217 2020/05/23 12:14:52 jsing Exp $ */
+/* $OpenBSD: ssl_lib.c,v 1.220 2020/08/11 18:39:40 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -241,7 +241,7 @@ SSL_CTX_set_ssl_version(SSL_CTX *ctx, const SSL_METHOD *meth)
 SSL *
 SSL_new(SSL_CTX *ctx)
 {
-	SSL	*s;
+	SSL *s;
 
 	if (ctx == NULL) {
 		SSLerrorx(SSL_R_NULL_SSL_CTX);
@@ -252,15 +252,10 @@ SSL_new(SSL_CTX *ctx)
 		return (NULL);
 	}
 
-	if ((s = calloc(1, sizeof(*s))) == NULL) {
-		SSLerrorx(ERR_R_MALLOC_FAILURE);
-		return (NULL);
-	}
-	if ((s->internal = calloc(1, sizeof(*s->internal))) == NULL) {
-		free(s);
-		SSLerrorx(ERR_R_MALLOC_FAILURE);
-		return (NULL);
-	}
+	if ((s = calloc(1, sizeof(*s))) == NULL)
+		goto err;
+	if ((s->internal = calloc(1, sizeof(*s->internal))) == NULL)
+		goto err;
 
 	s->internal->min_version = ctx->internal->min_version;
 	s->internal->max_version = ctx->internal->max_version;
@@ -942,10 +937,20 @@ SSL_is_server(const SSL *s)
 	return s->server;
 }
 
+static long
+ssl_get_default_timeout()
+{
+	/*
+	 * 2 hours, the 24 hours mentioned in the TLSv1 spec
+	 * is way too long for http, the cache would over fill.
+	 */
+	return (2 * 60 * 60);
+}
+
 long
 SSL_get_default_timeout(const SSL *s)
 {
-	return (s->method->internal->get_timeout());
+	return (ssl_get_default_timeout());
 }
 
 int
@@ -1752,7 +1757,7 @@ SSL_CTX_new(const SSL_METHOD *meth)
 	ret->internal->session_cache_tail = NULL;
 
 	/* We take the system default */
-	ret->session_timeout = meth->internal->get_timeout();
+	ret->session_timeout = ssl_get_default_timeout();
 
 	ret->internal->new_session_cb = 0;
 	ret->internal->remove_session_cb = 0;
@@ -2216,15 +2221,6 @@ SSL_set_ssl_method(SSL *s, const SSL_METHOD *meth)
 			ret = s->method->internal->ssl_new(s);
 		}
 
-		/*
-		 * XXX - reset the client max version to that of the incoming
-		 * method, otherwise a caller that uses a TLS_method() and then
-		 * sets with TLS_client_method() cannot do TLSv1.3.
-		 */
-		if (meth->internal->max_version == TLS1_3_VERSION &&
-		    meth->internal->ssl_connect != NULL)
-			s->internal->max_version = meth->internal->max_version;
-
 		if (conn == 1)
 			s->internal->handshake_func = meth->internal->ssl_connect;
 		else if (conn == 0)
