@@ -1,4 +1,4 @@
-/* $OpenBSD: ssl_pkt.c,v 1.24 2020/03/16 15:25:14 tb Exp $ */
+/* $OpenBSD: ssl_pkt.c,v 1.30 2020/08/09 16:54:16 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -617,48 +617,23 @@ ssl3_write_bytes(SSL *s, int type, const void *buf_, int len)
 }
 
 static int
-ssl3_create_record(SSL *s, unsigned char *p, int type, const unsigned char *buf,
-    unsigned int len)
+ssl3_create_record(SSL *s, CBB *cbb, uint16_t version, uint8_t type,
+    const unsigned char *buf, unsigned int len)
 {
 	SSL3_RECORD_INTERNAL *wr = &(S3I(s)->wrec);
 	SSL_SESSION *sess = s->session;
-	int eivlen, mac_size;
-	uint16_t version;
-	CBB cbb;
+	int block_size = 0, eivlen = 0, mac_size = 0;
+	size_t pad_len, record_len;
+	CBB fragment;
+	uint8_t *p;
 
-	memset(&cbb, 0, sizeof(cbb));
-
-	if ((sess == NULL) || (s->internal->enc_write_ctx == NULL) ||
-	    (EVP_MD_CTX_md(s->internal->write_hash) == NULL)) {
-		mac_size = 0;
-	} else {
-		mac_size = EVP_MD_CTX_size(s->internal->write_hash);
-		if (mac_size < 0)
+	if (sess != NULL && s->internal->enc_write_ctx != NULL &&
+	    EVP_MD_CTX_md(s->internal->write_hash) != NULL) {
+		if ((mac_size = EVP_MD_CTX_size(s->internal->write_hash)) < 0)
 			goto err;
 	}
 
-	/*
-	 * Some servers hang if initial client hello is larger than 256
-	 * bytes and record version number > TLS 1.0.
-	 */
-	version = s->version;
-	if (S3I(s)->hs.state == SSL3_ST_CW_CLNT_HELLO_B && !s->internal->renegotiate &&
-	    TLS1_get_version(s) > TLS1_VERSION)
-		version = TLS1_VERSION;
-
-	if (!CBB_init_fixed(&cbb, p, SSL3_RT_HEADER_LENGTH))
-		goto err;
-
-	/* Write the header. */
-	if (!CBB_add_u8(&cbb, type))
-		goto err;
-	if (!CBB_add_u16(&cbb, version))
-		goto err;
-
-	p += SSL3_RT_HEADER_LENGTH;
-
 	/* Explicit IV length. */
-	eivlen = 0;
 	if (s->internal->enc_write_ctx && SSL_USE_EXPLICIT_IV(s)) {
 		int mode = EVP_CIPHER_CTX_mode(s->internal->enc_write_ctx);
 		if (mode == EVP_CIPH_CBC_MODE) {
@@ -671,20 +646,37 @@ ssl3_create_record(SSL *s, unsigned char *p, int type, const unsigned char *buf,
 		eivlen = s->internal->aead_write_ctx->variable_nonce_len;
 	}
 
-	/* lets setup the record stuff. */
+	/* Determine length of record fragment. */
+	record_len = eivlen + len + mac_size;
+	if (s->internal->enc_write_ctx != NULL) {
+		block_size = EVP_CIPHER_CTX_block_size(s->internal->enc_write_ctx);
+		if (block_size <= 0 || block_size > EVP_MAX_BLOCK_LENGTH)
+			goto err;
+		if (block_size > 1) {
+			pad_len = block_size - (record_len % block_size);
+			record_len += pad_len;
+		}
+	} else if (s->internal->aead_write_ctx != NULL) {
+		record_len += s->internal->aead_write_ctx->tag_len;
+	}
+
+	/* Write the header. */
+	if (!CBB_add_u8(cbb, type))
+		goto err;
+	if (!CBB_add_u16(cbb, version))
+		goto err;
+	if (!CBB_add_u16_length_prefixed(cbb, &fragment))
+		goto err;
+	if (!CBB_add_space(&fragment, &p, record_len))
+		goto err;
+
+	/* Set up the record. */
 	wr->type = type;
 	wr->data = p + eivlen;
 	wr->length = (int)len;
-	wr->input = (unsigned char *)buf;
-
-	/* we now 'read' from wr->input, wr->length bytes into wr->data */
-
-	memcpy(wr->data, wr->input, wr->length);
 	wr->input = wr->data;
 
-	/* we should still have the output to wr->data and the input
-	 * from wr->input.  Length should be wr->length.
-	 * wr->data still points in the wb->buf */
+	memcpy(wr->data, buf, len);
 
 	if (mac_size != 0) {
 		if (tls1_mac(s, &(p[wr->length + eivlen]), 1) < 0)
@@ -692,66 +684,63 @@ ssl3_create_record(SSL *s, unsigned char *p, int type, const unsigned char *buf,
 		wr->length += mac_size;
 	}
 
-	wr->input = p;
 	wr->data = p;
+	wr->input = p;
+	wr->length += eivlen;
 
-	if (eivlen) {
-		/* if (RAND_pseudo_bytes(p, eivlen) <= 0)
-			goto err;
-		*/
-		wr->length += eivlen;
-	}
-
-	/* tls1_enc can only have an error on read */
-	tls1_enc(s, 1);
-
-	/* record length after mac and block padding */
-	if (!CBB_add_u16(&cbb, wr->length))
-		goto err;
-	if (!CBB_finish(&cbb, NULL, NULL))
+	if (tls1_enc(s, 1) != 1)
 		goto err;
 
-	/* we should now have
-	 * wr->data pointing to the encrypted data, which is
-	 * wr->length long */
+	if (wr->length != record_len)
+		goto err;
+
+	if (!CBB_flush(cbb))
+		goto err;
+
+	/*
+	 * We should now have wr->data pointing to the encrypted data,
+	 * which is wr->length long.
+	 */
 	wr->type = type; /* not needed but helps for debugging */
 	wr->length += SSL3_RT_HEADER_LENGTH;
 
 	return 1;
 
  err:
-	CBB_cleanup(&cbb);
-
 	return 0;
 }
 
 static int
 do_ssl3_write(SSL *s, int type, const unsigned char *buf, unsigned int len)
 {
-	SSL3_RECORD_INTERNAL *wr = &(S3I(s)->wrec);
 	SSL3_BUFFER_INTERNAL *wb = &(S3I(s)->wbuf);
 	SSL_SESSION *sess = s->session;
-	unsigned char *p;
-	int i, clear = 0;
-	int prefix_len = 0;
-	size_t align;
+	int need_empty_fragment = 0;
+	size_t align, out_len;
+	uint16_t version;
+	CBB cbb;
+	int ret;
+
+	memset(&cbb, 0, sizeof(cbb));
 
 	if (wb->buf == NULL)
 		if (!ssl3_setup_write_buffer(s))
 			return -1;
 
-	/* first check if there is a SSL3_BUFFER_INTERNAL still being written
-	 * out.  This will happen with non blocking IO */
+	/*
+	 * First check if there is a SSL3_BUFFER_INTERNAL still being written
+	 * out.  This will happen with non blocking IO.
+	 */
 	if (wb->left != 0)
 		return (ssl3_write_pending(s, type, buf, len));
 
-	/* If we have an alert to send, lets send it */
+	/* If we have an alert to send, let's send it. */
 	if (S3I(s)->alert_dispatch) {
-		i = s->method->ssl_dispatch_alert(s);
-		if (i <= 0)
-			return (i);
-		/* if it went, fall through and send more stuff */
-		/* we may have released our buffer, so get it again */
+		if ((ret = s->method->ssl_dispatch_alert(s)) <= 0)
+			return (ret);
+		/* If it went, fall through and send more stuff. */
+
+		/* We may have released our buffer, if so get it again. */
 		if (wb->buf == NULL)
 			if (!ssl3_setup_write_buffer(s))
 				return -1;
@@ -760,67 +749,71 @@ do_ssl3_write(SSL *s, int type, const unsigned char *buf, unsigned int len)
 	if (len == 0)
 		return 0;
 
-	align = (size_t)wb->buf + SSL3_RT_HEADER_LENGTH;
-	align = (-align) & (SSL3_ALIGN_PAYLOAD - 1);
+	/*
+	 * Some servers hang if initial client hello is larger than 256
+	 * bytes and record version number > TLS 1.0.
+	 */
+	version = s->version;
+	if (S3I(s)->hs.state == SSL3_ST_CW_CLNT_HELLO_B && !s->internal->renegotiate &&
+	    TLS1_get_version(s) > TLS1_VERSION)
+		version = TLS1_VERSION;
 
-	p = wb->buf + align;
-	wb->offset = align;
-
-	if ((sess == NULL) || (s->internal->enc_write_ctx == NULL) ||
-	    (EVP_MD_CTX_md(s->internal->write_hash) == NULL)) {
-		clear = s->internal->enc_write_ctx ? 0 : 1; /* must be AEAD cipher */
+	/*
+	 * Countermeasure against known-IV weakness in CBC ciphersuites
+	 * (see http://www.openssl.org/~bodo/tls-cbc.txt). Note that this
+	 * is unnecessary for AEAD.
+	 */
+	if (sess != NULL && s->internal->enc_write_ctx != NULL &&
+	    EVP_MD_CTX_md(s->internal->write_hash) != NULL) {
+		if (S3I(s)->need_empty_fragments &&
+		    !S3I(s)->empty_fragment_done &&
+		    type == SSL3_RT_APPLICATION_DATA)
+			need_empty_fragment = 1;
 	}
 
-	if (!clear && !S3I(s)->empty_fragment_done) {
-		/*
-		 * Countermeasure against known-IV weakness in CBC ciphersuites
-		 * (see http://www.openssl.org/~bodo/tls-cbc.txt)
-		 */
-		if (S3I(s)->need_empty_fragments &&
-		    type == SSL3_RT_APPLICATION_DATA) {
-			/* extra fragment would be couple of cipher blocks,
-			 * which would be multiple of SSL3_ALIGN_PAYLOAD, so
-			 * if we want to align the real payload, then we can
-			 * just pretent we simply have two headers. */
-			align = (size_t)wb->buf + 2 * SSL3_RT_HEADER_LENGTH;
-			align = (-align) & (SSL3_ALIGN_PAYLOAD - 1);
+	/*
+	 * An extra fragment would be a couple of cipher blocks, which would
+	 * be a multiple of SSL3_ALIGN_PAYLOAD, so if we want to align the real
+	 * payload, then we can just simply pretend we have two headers.
+	 */
+	align = (size_t)wb->buf + SSL3_RT_HEADER_LENGTH;
+	if (need_empty_fragment)
+		align += SSL3_RT_HEADER_LENGTH;
+	align = (-align) & (SSL3_ALIGN_PAYLOAD - 1);
+	wb->offset = align;
 
-			p = wb->buf + align;
-			wb->offset = align;
-
-			if (!ssl3_create_record(s, p, type, buf, 0))
-				goto err;
-
-			prefix_len = wr->length;
-			if (prefix_len > (SSL3_RT_HEADER_LENGTH +
-			    SSL3_RT_SEND_MAX_ENCRYPTED_OVERHEAD)) {
-				/* insufficient space */
-				SSLerror(s, ERR_R_INTERNAL_ERROR);
-				goto err;
-			}
-
-			p = wb->buf + wb->offset + prefix_len;
-		}
+	if (!CBB_init_fixed(&cbb, wb->buf + align, wb->len - align))
+		goto err;
 
+	if (need_empty_fragment) {
+		if (!ssl3_create_record(s, &cbb, version, type, buf, 0))
+			goto err;
 		S3I(s)->empty_fragment_done = 1;
 	}
 
-	if (!ssl3_create_record(s, p, type, buf, len))
+	if (!ssl3_create_record(s, &cbb, version, type, buf, len))
 		goto err;
 
-	/* now let's set up wb */
-	wb->left = prefix_len + wr->length;
+	if (!CBB_finish(&cbb, NULL, &out_len))
+		goto err;
 
-	/* memorize arguments so that ssl3_write_pending can detect
-	 * bad write retries later */
+	wb->left = out_len;
+
+	/*
+	 * Memorize arguments so that ssl3_write_pending can detect
+	 * bad write retries later.
+	 */
 	S3I(s)->wpend_tot = len;
 	S3I(s)->wpend_buf = buf;
 	S3I(s)->wpend_type = type;
 	S3I(s)->wpend_ret = len;
 
-	/* we now just need to write the buffer */
+	/* We now just need to write the buffer. */
 	return ssl3_write_pending(s, type, buf, len);
-err:
+
+ err:
+	CBB_cleanup(&cbb);
+
 	return -1;
 }
 
@@ -843,9 +836,8 @@ ssl3_write_pending(SSL *s, int type, const unsigned char *buf, unsigned int len)
 		errno = 0;
 		if (s->wbio != NULL) {
 			s->internal->rwstate = SSL_WRITING;
-			i = BIO_write(s->wbio,
-			(char *)&(wb->buf[wb->offset]),
-			(unsigned int)wb->left);
+			i = BIO_write(s->wbio, (char *)&(wb->buf[wb->offset]),
+			    (unsigned int)wb->left);
 		} else {
 			SSLerror(s, SSL_R_BIO_NOT_SET);
 			i = -1;
